@@ -32,7 +32,8 @@ enum {
 	PROP_HTML_MODE,
 	PROP_INLINE_SPELLING,
 	PROP_MAGIC_LINKS,
-	PROP_MAGIC_SMILEYS
+	PROP_MAGIC_SMILEYS,
+	PROP_CHANGED
 };
 
 enum {
@@ -216,6 +217,12 @@ editor_url_requested_cb (GtkhtmlEditor *editor,
 	g_signal_emit (editor, signals[URI_REQUESTED], 0, url, stream);
 }
 
+static void
+engine_undo_changed_cb (GtkhtmlEditor *editor, HTMLEngine *engine)
+{
+	g_object_notify (G_OBJECT (editor), "changed");
+}
+
 static void
 editor_command_popup_menu (GtkhtmlEditor *editor)
 {
@@ -500,6 +507,10 @@ editor_constructor (GType type,
 		html, "url_requested",
 		G_CALLBACK (editor_url_requested_cb), editor);
 
+	g_signal_connect_swapped (
+		html->engine, "undo-changed",
+		G_CALLBACK (engine_undo_changed_cb), editor);
+
 	/* Connect property dialog widgets to actions. */
 
 	gtk_activatable_set_related_action (
@@ -644,6 +655,11 @@ editor_set_property (GObject *object,
 				GTKHTML_EDITOR (object),
 				g_value_get_boolean (value));
 			return;
+		case PROP_CHANGED:
+			gtkhtml_editor_set_changed (
+				GTKHTML_EDITOR (object),
+				g_value_get_boolean (value));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -697,6 +713,11 @@ editor_get_property (GObject *object,
 				value, gtkhtml_editor_get_magic_smileys (
 				GTKHTML_EDITOR (object)));
 			return;
+		case PROP_CHANGED:
+			g_value_set_boolean (
+				value, gtkhtml_editor_get_changed (
+				GTKHTML_EDITOR (object)));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -726,6 +747,23 @@ editor_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static gboolean
+editor_key_press_event (GtkWidget *widget,
+                        GdkEventKey *event)
+{
+#ifdef HAVE_XFREE
+	GtkhtmlEditor *editor = GTKHTML_EDITOR (widget);
+
+	if (event->keyval == XF86XK_Spell) {
+		gtk_action_activate (ACTION (SPELL_CHECK));
+		return TRUE;
+	}
+#endif
+
+	/* Chain up to parent's key_press_event() method. */
+	return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+}
+
 static void
 editor_cut_clipboard (GtkhtmlEditor *editor)
 {
@@ -754,6 +792,7 @@ static void
 editor_class_init (GtkhtmlEditorClass *class)
 {
 	GObjectClass *object_class;
+	GtkWidgetClass *widget_class;
 
 	parent_class = g_type_class_peek_parent (class);
 	g_type_class_add_private (class, sizeof (GtkhtmlEditorPrivate));
@@ -765,6 +804,9 @@ editor_class_init (GtkhtmlEditorClass *class)
 	object_class->dispose = editor_dispose;
 	object_class->finalize = editor_finalize;
 
+	widget_class = GTK_WIDGET_CLASS (class);
+	widget_class->key_press_event = editor_key_press_event;
+
 	class->cut_clipboard = editor_cut_clipboard;
 	class->copy_clipboard = editor_copy_clipboard;
 	class->paste_clipboard = editor_paste_clipboard;
@@ -775,8 +817,8 @@ editor_class_init (GtkhtmlEditorClass *class)
 		PROP_CURRENT_FOLDER,
 		g_param_spec_string (
 			"current-folder",
-			_("Current Folder"),
-			_("The initial folder for file chooser dialogs"),
+			"Current Folder",
+			"The initial folder for file chooser dialogs",
 			g_get_home_dir (),
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT));
@@ -786,8 +828,8 @@ editor_class_init (GtkhtmlEditorClass *class)
 		PROP_FILENAME,
 		g_param_spec_string (
 			"filename",
-			_("Filename"),
-			_("The filename to use when saving"),
+			"Filename",
+			"The filename to use when saving",
 			NULL,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT));
@@ -797,8 +839,8 @@ editor_class_init (GtkhtmlEditorClass *class)
 		PROP_HTML,
 		g_param_spec_object (
 			"html",
-			_("HTML Editing Widget"),
-			_("The main HTML editing widget"),
+			"HTML Editing Widget",
+			"The main HTML editing widget",
 			GTK_TYPE_HTML,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT_ONLY));
@@ -808,8 +850,8 @@ editor_class_init (GtkhtmlEditorClass *class)
 		PROP_HTML_MODE,
 		g_param_spec_boolean (
 			"html-mode",
-			_("HTML Mode"),
-			_("Edit HTML or plain text"),
+			"HTML Mode",
+			"Edit HTML or plain text",
 			TRUE,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT));
@@ -819,8 +861,8 @@ editor_class_init (GtkhtmlEditorClass *class)
 		PROP_INLINE_SPELLING,
 		g_param_spec_boolean (
 			"inline-spelling",
-			_("Inline Spelling"),
-			_("Check your spelling as you type"),
+			"Inline Spelling",
+			"Check your spelling as you type",
 			TRUE,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT));
@@ -830,8 +872,8 @@ editor_class_init (GtkhtmlEditorClass *class)
 		PROP_MAGIC_LINKS,
 		g_param_spec_boolean (
 			"magic-links",
-			_("Magic Links"),
-			_("Make URIs clickable as you type"),
+			"Magic Links",
+			"Make URIs clickable as you type",
 			TRUE,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT));
@@ -841,12 +883,22 @@ editor_class_init (GtkhtmlEditorClass *class)
 		PROP_MAGIC_SMILEYS,
 		g_param_spec_boolean (
 			"magic-smileys",
-			_("Magic Smileys"),
-			_("Convert emoticons to images as you type"),
+			"Magic Smileys",
+			"Convert emoticons to images as you type",
 			TRUE,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_CHANGED,
+		g_param_spec_boolean (
+			"changed",
+			_("Changed property"),
+			_("Whether editor changed"),
+			FALSE,
+			G_PARAM_READWRITE));
+
 	signals[COMMAND_AFTER] = g_signal_new (
 		"command-after",
 		G_OBJECT_CLASS_TYPE (class),
@@ -1108,6 +1160,8 @@ gtkhtml_editor_set_changed (GtkhtmlEditor *editor,
 	}
 
 	editor->priv->changed = changed;
+
+	g_object_notify (G_OBJECT (editor), "changed");
 }
 
 const gchar *
@@ -1726,7 +1780,7 @@ gtkhtml_editor_undo_end (GtkhtmlEditor *editor)
 
 	html = gtkhtml_editor_get_html (editor);
 
-	html_undo_level_end (html->engine->undo);
+	html_undo_level_end (html->engine->undo, html->engine);
 }
 
 gboolean
