@@ -109,6 +109,7 @@ face_tool_button_reposition_window (GtkhtmlFaceToolButton *button)
 	GdkScreen *screen;
 	GdkWindow *window;
 	GdkRectangle monitor;
+	GtkAllocation allocation;
 	gint monitor_num;
 	gint x, y, width, height;
 
@@ -119,13 +120,15 @@ face_tool_button_reposition_window (GtkhtmlFaceToolButton *button)
 
 	gdk_window_get_origin (window, &x, &y);
 
-	if (GTK_WIDGET_NO_WINDOW (button)) {
-		x += GTK_WIDGET (button)->allocation.x;
-		y += GTK_WIDGET (button)->allocation.y;
+	if (!gtk_widget_get_has_window (GTK_WIDGET (button))) {
+		gtk_widget_get_allocation (GTK_WIDGET (button), &allocation);
+		x += allocation.x;
+		y += allocation.y;
 	}
 
-	width = button->priv->window->allocation.width;
-	height = button->priv->window->allocation.height;
+	gtk_widget_get_allocation (button->priv->window, &allocation);
+	width = allocation.width;
+	height = allocation.height;
 
 	x = CLAMP (x, monitor.x, monitor.x + monitor.width - width);
 	y = CLAMP (y, monitor.y, monitor.y + monitor.height - height);
@@ -146,7 +149,11 @@ face_tool_button_face_release_event_cb (GtkhtmlFaceToolButton *button,
                                         GdkEventButton *event,
                                         GtkButton *face_button)
 {
-	if (GTK_WIDGET_STATE (button) != GTK_STATE_NORMAL)
+	GtkStateType state;
+
+	state = gtk_widget_get_state (GTK_WIDGET (button));
+
+	if (state != GTK_STATE_NORMAL)
 		gtk_button_clicked (face_button);
 
 	return FALSE;
@@ -324,7 +331,7 @@ face_tool_button_popup (GtkhtmlFaceToolButton *button)
 	GdkWindow *window;
 	GdkGrabStatus status;
 
-	if (!GTK_WIDGET_REALIZED (button))
+	if (!gtk_widget_get_realized (GTK_WIDGET (button)))
 		return;
 
 	if (button->priv->popup_shown)
@@ -366,7 +373,7 @@ face_tool_button_popdown (GtkhtmlFaceToolButton *button)
 {
 	GtkToggleToolButton *tool_button;
 
-	if (!GTK_WIDGET_REALIZED (button))
+	if (!gtk_widget_get_realized (GTK_WIDGET (button)))
 		return;
 
 	if (!button->priv->popup_shown)
@@ -522,7 +529,7 @@ face_tool_button_init (GtkhtmlFaceToolButton *button)
 	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
 	gtk_window_set_type_hint (
 		GTK_WINDOW (window), GDK_WINDOW_TYPE_HINT_COMBO);
-	if (GTK_WIDGET_TOPLEVEL (toplevel)) {
+	if (gtk_widget_is_toplevel (toplevel)) {
 		gtk_window_group_add_window (
 			gtk_window_get_group (GTK_WINDOW (toplevel)),
 			GTK_WINDOW (window));
