@@ -46,7 +46,7 @@ get_locale_language (void)
 	locale = g_win32_getlocale ();
 #endif
 	if (locale && strcmp (locale, "C") && strcmp (locale, "POSIX")) {
-		char *codeset, *lang;
+		gchar *codeset, *lang;
 
 		if ((codeset = strchr (locale, '.')))
 			lang = g_strndup (locale, codeset - locale);
@@ -57,17 +57,17 @@ get_locale_language (void)
 		if (strlen (lang) >= 2) {
 			if (lang[2] == '-' || lang[2] == '_') {
 				/* canonicalise the lang */
-				lang [0] = g_ascii_tolower (lang [0]);
-				lang [1] = g_ascii_tolower (lang [1]);
+				lang[0] = g_ascii_tolower (lang[0]);
+				lang[1] = g_ascii_tolower (lang[1]);
 
 				/* validate the country code */
 				if (strlen (lang + 3) > 2) {
-				        /* invalid country code */
+					/* invalid country code */
 					lang[2] = '\0';
 				} else {
 					lang[2] = '-';
-					lang [3] = g_ascii_toupper (lang [3]);
-					lang [4] = g_ascii_toupper (lang [4]);
+					lang[3] = g_ascii_toupper (lang[3]);
+					lang[4] = g_ascii_toupper (lang[4]);
 				}
 			} else if (lang[2] != '\0') {
 				/* invalid language */
@@ -86,7 +86,7 @@ get_locale_language (void)
 }
 
 GtkHTMLClassProperties *
-gtk_html_class_properties_new (GtkWidget *widget)
+gtk_html_class_properties_new (void)
 {
 	GtkHTMLClassProperties *p = g_new0 (GtkHTMLClassProperties, 1);
 
@@ -102,25 +102,9 @@ gtk_html_class_properties_destroy (GtkHTMLClassProperties *p)
 	g_free (p);
 }
 
-#define GNOME_SPELL_GCONF_DIR "/GNOME/Spell"
-#define GETSP(t,x,prop,f,c) \
-        key = g_strconcat (GNOME_SPELL_GCONF_DIR, x, NULL); \
-        val = gconf_client_get_without_default (client, key, NULL); \
-        if (val) { f; p->prop = c (gconf_value_get_ ## t (val)); \
-        gconf_value_free (val); } \
-        g_free (key);
-
 void
 gtk_html_class_properties_load (GtkHTMLClassProperties *p, GConfClient *client)
 {
-	GConfValue *val;
-	gchar *key;
-
-	g_assert (client);
-
-	GETSP (string, "/language", language,
-	       g_free (p->language), g_strdup);
-
 }
 
 #define SET(t,x,prop) \
@@ -128,7 +112,6 @@ gtk_html_class_properties_load (GtkHTMLClassProperties *p, GConfClient *client)
         gconf_client_set_ ## t (client, key, p->prop, NULL); \
         g_free (key); }
 
-
 void
 gtk_html_class_properties_update (GtkHTMLClassProperties *p, GConfClient *client, GtkHTMLClassProperties *old)
 {
@@ -311,6 +294,7 @@ static GEnumValue _gtk_html_command_values[] = {
 	{ GTK_HTML_COMMAND_IS_SELECTION_ACTIVE, "GTK_HTML_COMMAND_IS_SELECTION_ACTIVE", "is-selection-active" },
 	{ GTK_HTML_COMMAND_SCROLL_FORWARD, "GTK_HTML_COMMAND_SCROLL_FORWARD", "scroll-forward" },
 	{ GTK_HTML_COMMAND_SCROLL_BACKWARD, "GTK_HTML_COMMAND_SCROLL_BACKWARD", "scroll-backward" },
+	{ GTK_HTML_COMMAND_UNSELECT_ALL, "GTK_HTML_COMMAND_UNSELECT_ALL", "unselect-all" },
 	{ 0, NULL, NULL }
 };
 
