@@ -83,10 +83,10 @@ __inline
 inline
 #endif
 #endif
-static unsigned int
-hash (register const char *str, register unsigned int len)
+static guint
+hash (register const gchar *str, register guint len)
 {
-  static unsigned short asso_values[] =
+  static gushort asso_values[] =
     {
       739, 739, 739, 739, 739, 739, 739, 739, 739, 739,
       739, 739, 739, 739, 739, 739, 739, 739, 739, 739,
@@ -115,25 +115,25 @@ hash (register const char *str, register unsigned int len)
       739, 739, 739, 739, 739, 739, 739, 739, 739, 739,
       739, 739, 739, 739, 739, 739, 739
     };
-  register int hval = len;
+  register gint hval = len;
 
   switch (hval)
     {
       default:
-        hval += asso_values[(unsigned char)str[4]];
+        hval += asso_values[(guchar)str[4]];
       /*FALLTHROUGH*/
       case 4:
       case 3:
-        hval += asso_values[(unsigned char)str[2]];
+        hval += asso_values[(guchar)str[2]];
       /*FALLTHROUGH*/
       case 2:
-        hval += asso_values[(unsigned char)str[1]+1];
+        hval += asso_values[(guchar)str[1]+1];
       /*FALLTHROUGH*/
       case 1:
-        hval += asso_values[(unsigned char)str[0]];
+        hval += asso_values[(guchar)str[0]];
         break;
     }
-  return hval + asso_values[(unsigned char)str[len - 1]];
+  return hval + asso_values[(guchar)str[len - 1]];
 }
 
 #ifdef __GNUC__
@@ -143,9 +143,9 @@ __attribute__ ((__gnu_inline__))
 #endif
 #endif
 static struct _EntityEntry *
-html_entity_hash (register const char *str, register unsigned int len)
+html_entity_hash (register const gchar *str, register guint len)
 {
-  static unsigned char lengthtable[] =
+  static guchar lengthtable[] =
     {
        0,  0,  0,  0,  0,  0,  0,  0,  3,  0,  0,  0,  0,  3,
        0,  0,  0,  0,  3,  0,  0,  0,  0,  0,  4,  0,  0,  0,
@@ -850,12 +850,12 @@ html_entity_hash (register const char *str, register unsigned int len)
 
   if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
     {
-      register int key = hash (str, len);
+      register gint key = hash (str, len);
 
       if (key <= MAX_HASH_VALUE && key >= 0)
         if (len == lengthtable[key])
           {
-            register const char *s = wordlist[key].name;
+            register const gchar *s = wordlist[key].name;
 
             if (*str == *s && !memcmp (str + 1, s + 1, len - 1))
               return &wordlist[key];
